function [SMP,new_corr_mat] = generateUnifRV(dim, corr_mat, num_sampl),

% [SMP,new_corr_mat] = generateUnifRV(dim, Rcorr_mat,num_sampl),
% returns "num_sampl" samples of "dim"-dimensioned 
% UNIFORM random vectors given a correlation matrix "corr_mat".
%
% The output is in the form of a matrix with "num_sampl" rows 
% and "dim" columns.
%
% In case the given corr_mat turns out NORTA defective, an
% SDP adjustment is done, and new_corr_mat gives the best
% corr_mat estimated using this method of NORTA-correction.
%
% Note that if you wanted arbitrary CONTINUOUS marginals with
% a rank-correlation structure, then input the rank-correlation 
% matrix as the "corr_mat" and then do a component-wise inverse 
% on the output samples to get the right marginals.
% 

corr_Z=2*sin(pi/6*corr_mat);                % NORTA Transform
eig_corr_Z=eig(corr_Z);                    


% SDP CORRECTION: COMMENT OUT IF UNDESIRED
% The following code applies if the NORTA-transform is not pd    
if min(eig_corr_Z)<0,                     

    % readjusts corr_Z using an L1 SDP formulation
    % corr_Z=mkCorrUsingSDPL1(corr_Z);

    % OR readjusts corr_Z using an L1 SDP formulation w/ bound on Linf change
    % corr_Z=mkCorrUsingSDPL1wLinfBd(corr_Z,Linf_Bd);

    % OR readjusts corr_Z using an Linf SDP formulation
    % corr_Z=mkCorrUsingSDPLinf(corr_Z);

    % OR readjusts corr_Z using an Eigenvalue Correction heuristic
    corr_Z=mkCorrUsingEC(corr_Z);

end

SMP =randn(num_sampl,dim);
chol_corr_Z=chol(corr_Z);

SMP=SMP*chol_corr_Z;  % this gives the samples the right corr_Z

SMP=normcdf(SMP);     % this gives the samples uniform CDFs
                      % note that to give other CDFs one has to
                      % now apply further (inverse) tranformations 
                      % to the uniform CDF'ed samples.

new_corr_mat=6*asin(corr_Z/2)/pi;  % the best corr_mat got if the 
                                   % original corr_mat wasn't good enuf
    



