function newA = mkCorrUsingEC(A)

% This function implements the Eigenvalue Correction (EC) method.
%
% Returns a matrix newA that has been adjusted to be positive semidefinite,
% given an indefinite matrix A. So, matrix A is in the correlation in 
% NORMALs space, i.e., the matrix used as the base for the NORTA transformation.
% The way we do this is
% that we find out the matrix factorization of A and change any negative
% eigenvalues to EPS, a matlab-defined small positive constant.
% Now this usually results in a matrix that doesn't have 1's
% in the diagonal. So, this is adjusted for by pre and post multiplying with
% a diagonal matrix that scales the specific rows and columns which have a 
% non-1 diagonal entry.
%

%Eigval decomp
[AEVCS,AED] = eig(A);

dim=size(AEVCS,1); 
eigs=AED*ones(dim,1);

% index of good eigvals
goodeigs=(eigs>=eps); 

% zero out neg eigvals and then make them of value EPS
rectified_eigs=eigs.*goodeigs+(1-goodeigs)*eps; 
goodD=diag(rectified_eigs);
newA=AEVCS*goodD*AEVCS';

% find the new diag elements. Need to make them 1!
newDiags=diag(newA);
corrD=diag(1./sqrt(newDiags));
newA=corrD*newA*corrD;
