function newGood = mkCorrUsingSDPLinf(A)

% newGood = mkCorrUsingSDPLinf(A) 
% A is the Norta Defective sym. matrix that needs to be rectified, and
% sets up the SDP problem to minimize the L_inf distance. 
%
% Returned:
% newGood - the new good matrix in NORMAL space

%    copt - the optimal objective value for the solution
%    xopt - the optimal solution

dim = size(A,1);

X1=zeros(dim,dim);  % sets up var count for the LMI declarations
X2=zeros(dim,dim);

numvar=0;
for i= 1:dim-1,
    for j= i+1:dim,
        X1(i,j)=numvar+1;
        X2(i,j)=numvar+2;
        numvar=numvar+2;
        
        % sets up X1 as [0 1 3 5; 0 0 7 9 ; 0 0 0 11; 0 0 0 0]
        % sets up X2 as [0 2 4 6; 0 0 8 10; 0 0 0 12; 0 0 0 0]
    end
end
X1=X1+X1';  % mirrors them to get symm matrices
X2=X2+X2';

setlmis([]);
sdpX1=lmivar(3,X1);
sdpX2=lmivar(3,X2);

sdpY=lmivar(3,diag([1:numvar])) ;

sdpZ1=lmivar(3,diag(2*[1:numvar/2]'-ones(numvar/2,1)));
sdpZ2=lmivar(3,diag(2*[1:numvar/2]));        % numvar is EVEN
sdpZ=lmivar(3,(numvar+1)*eye(numvar/2));

lmiterm([-1 1 1 sdpX1],1,1);                 % LMI #1: pos change
lmiterm([ 1 1 1 sdpX2],1,1);                 % LMI #1: neg change
lmiterm([-1 1 1 0],A);                       % LMI #1: ND matrix
lmiterm([-2 1 1 sdpY],1,1);                  % LMI #2: vars >=0
lmiterm([ 3 1 1 sdpZ1],1,1);                 % LMI #3: abs(change)<=L_inf, pos
lmiterm([ 3 1 1 sdpZ2],1,1);                 % LMI #3: abs(change)<=L_inf, neg
lmiterm([-3 1 1 sdpZ],1,1);                  % LMI #3: pos+neg-L_inf<=0

temp=getlmis;                                % name of LMI system
C=[zeros(numvar,1); 1];
[copt,xopt]=mincx(temp,C,[.00001 0 0 0 1 ]); % third input is options,
                                             % and the only changes are
                                             % to the relative precision,
                                             % and to printing.

% decinfo(temp)                                % to check the formaulation
% keyboard;
newGood = A + dec2mat(temp,xopt,1) - dec2mat(temp,xopt,2);

%if sum(eig(newGood<0)>0,
%    error('grave error1');
%end

return
